#!/bin/bash

IMXBOOT_DIR=$(dirname "$(realpath "$0")")
SECUREBOOT_DIR=$(realpath "$IMXBOOT_DIR/../secureboot_x2")
[ -e "$IMXBOOT_DIR/secureboot/a900_helper.sh" ] && SECUREBOOT_DIR=$(realpath "$IMXBOOT_DIR/../secureboot_a900")
# keep SETUP_DIR for compat with older secureboot.conf (easier copy/paste)
SETUP_DIR="$IMXBOOT_DIR"

if [ -e "$SECUREBOOT_DIR/secureboot.conf" ]; then
	. "$SECUREBOOT_DIR/secureboot.conf"
elif [ -e "$IMXBOOT_DIR/secureboot.conf" ]; then
	. "$IMXBOOT_DIR/secureboot.conf"
else
	. "$IMXBOOT_DIR/secureboot.conf.example"
fi

# some defaults...
SWU_OUT_DIR=${SWU_OUT_DIR:-$SECUREBOOT_DIR/swu}
OUT=${OUT:-$SECUREBOOT_DIR/out}
CST=${CST:-$SECUREBOOT_DIR/cst}
IMXBOOT=${IMXBOOT:-$IMXBOOT_DIR}
# variables used in swu desc files need to be exported
# in particular SECUREBOOT_OUT is only used for mkswu
export TMPDIR=${TMPDIR:-$SECUREBOOT_DIR/tmp}
export SECUREBOOT_OUT="$OUT"
export IMXBOOT_TARGET="$IMXBOOT_TARGET"

LOG_REDIR=""

case "$IMXBOOT_TARGET" in
armadillo-900) . "$IMXBOOT_DIR/secureboot/a900_helper.sh";;
*) . "$IMXBOOT_DIR/secureboot/x2_helper.sh";;
esac

print_hint() {
	local rc="$?"
	if [ "$rc" != 0 ]; then
		to_stderr printf "%s\n" "" \
			"Command failed ($rc)." \
			"Please see $LOG_REDIR for details"
	fi
	return $rc
}

redirect_logs() {
	[ -n "$LOG_REDIR" ] && return

	trap print_hint EXIT

	# - prints to fd 3 will be displayed as stdout, fd 4 for stderr
	# - all other logs will go to log file instead
	LOG_REDIR="$TMPDIR/$cmd.log"
	echo "Logging build outputs to $LOG_REDIR"
	echo
	exec 4>&2 3>&1 >"$LOG_REDIR" 2>&1
}

to_stderr() {
	if [ -n "$LOG_REDIR" ]; then
		"$@" 2>&1 | tee /proc/$$/fd/4 >&2
	else
		"$@" >&2
	fi
}

to_stdout_only() {
	if [ -n "$LOG_REDIR" ]; then
		"$@" >&3 2>&1
	else
		"$@" 2>&1
	fi
}

to_stdout() {
	if [ -n "$LOG_REDIR" ]; then
		"$@" 2>&1 | tee /proc/$$/fd/3
	else
		"$@" 2>&1
	fi
}

error() {
	to_stderr printf "ERROR: %s\n" "$@"
	exit 1
}

warning() {
	[ "${debug:-2}" -lt 1 ] && return
	to_stderr printf "WARNING: %s\n" "$@"
}

info() {
	to_stdout printf "%s\n" "$@"
}

ERRORS=""
soft_error() {
	warning "$@"
	ERRORS="$ERRORS
$*"
	return 1
}

prompt_yesno() {
	local default="$1"
	shift 1
	local yesno

	while true; do
		to_stdout_only read -r -p "$1" yesno
		[ -n "$yesno" ] || yesno="$default"
		case "$yesno" in
			[Yy] | [Yy][Ee][Ss] | [Tt][Rr][Uu][Ee] | 1)
				return 0
				;;
			[Nn] | [Nn][Oo] | [Ff][Aa][Ll][Ss][Ee] | 0)
				return 1
				;;
		esac
	done
}

pre_checks() {
	local overlay cmd="$1"
	[ -n "$OUT" ] || error "\$OUT is not set"
	[ -n "$CST" ] || error "\$CST is not set"
	[ -n "$SWU_OUT_DIR" ] || error "\$SWU_OUT_DIR is not set"
	[ -n "$TMPDIR" ] || error "\$TMPDIR is not set"

	mkdir -p "$OUT" "$CST" "$SWU_OUT_DIR" "$TMPDIR" || exit

	# Special commands that need to skip checks
	case "$cmd" in
	setup|print_srk_fuse) return;;
	esac

	if [ -e "$CST/linux64/bin/cst" ]; then
		CST_BIN=$CST/linux64/bin/cst
	elif command -v cst >/dev/null; then
		CST_BIN=cst
	else
		error "Please run $0 setup"
	fi

	case "$cmd" in
	secureboot_init|build|make_installer)
		# check setup environment
		[[ -e "$SECUREBOOT_DIR/build-rootfs" ]] \
			|| soft_error "could not find build-rootfs"
		[[ -e "$SECUREBOOT_DIR/imx-boot" ]] \
			|| soft_error "could not find imx_boot"

		command -v mkswu > /dev/null || soft_error "could not find mkswu command"

		[[ -z "$ERRORS" ]] || error "Please run $0 setup"

		if [[ "$(realpath "$SECUREBOOT_DIR/secureboot.sh")" != "$(realpath "$IMXBOOT_DIR/secureboot.sh")" ]]; then
			info "This script is not $(readlink "$SECUREBOOT_DIR/secureboot.sh") that was used during setup."
			prompt_yesno y "Update ? [Y/n]" || error "Aborted by user"
			ln -sf "$IMXBOOT_DIR/secureboot.sh" "$SECUREBOOT_DIR/secureboot.sh"
			ln -sf "$IMXBOOT_DIR" "$SECUREBOOT_DIR/imx-boot"
		fi
		[[ -e "$IMXBOOT" ]] || error "\$IMXBOOT does not exist: $IMXBOOT. Remove variable assignment?"
		check_or_download_linux
		;;
	imxboot*)
		check_cst_srk
		[[ -e "$IMXBOOT" ]] || error "\$IMXBOOT does not exist: $IMXBOOT. Remove variable assignment?"
		;;
	linux*)
		check_cst_srk
		check_or_download_linux
		;;
	esac

	redirect_logs
}

linux_apply_overlays() {
	if [[ "${#LINUX_DTB_OVERLAYS_PREAPPLY[@]}" = 0 ]]; then
		cp "$LINUX_DTB" "$TMPDIR/$dtb" || error "Could not copy dtb"
		return
	fi

	# add prefix
	local overlay
	local -a overlays
	for overlay in "${LINUX_DTB_OVERLAYS_PREAPPLY[@]}"; do
		if [[ "${overlay#/}" != "$overlay" ]]; then
			overlays+=( "$overlay" )
		else
			overlays+=( "${LINUX_DTB%/*}/$overlay" )
		fi
	done

	fdtoverlay -v -i "$LINUX_DTB" "${overlays[@]}" \
			-o "$TMPDIR/$dtb" \
		|| error "Could not apply overlays"
}

maybe_compress() {
	local file
	for file; do
		[[ -e "$file" ]] || continue
		[[ -s "$file.zst" ]] && [[ "$file.zst" -nt "$file" ]] && continue
		zstd -c "$file" > "$file.zst" \
			|| error "Could not compress $file"
	done
}

# helper: build fit image from linux image and dtbs
linux_build_fit() {
	local overlay

	MKIMAGE="$IMXBOOT/uboot-imx/$IMXBOOT_TARGET/tools/mkimage"
	if ! [ -e "$MKIMAGE" ]; then
		if command -v mkimage >/dev/null; then
			MKIMAGE=mkimage
		else
			error "mkimage not found, build imxboot first"
		fi
	fi

	# Note: each entry could have a 'hash-1 { algo = "sha1"; };' block, but
	# since we sign content it is redundant.
	# The hash might also not be compatible with encryption as we might
	# only encrypt the content and verification would happen too early
	{
		cat <<EOF
/dts-v1/;
/ {
	description = "fit linux image for armadillo";
	#address-cells = <1>;
	images {
		kernel {
			description = "linux kernel";
			data = /incbin/("$LINUX_IMAGE.zst");
			type = "kernel";
			arch = "arm64";
			os = "linux";
			compression = "zstd";
			load = <$BOARD_LINUX_LOAD_ADDR>;
			entry = <$BOARD_LINUX_LOAD_ADDR>;
		};
		fdt {
			description = "fdt";
			data = /incbin/("$TMPDIR/$dtb.zst");
			type = "flat_dt";
			arch = "arm64";
			compression = "zstd";
			load = <$BOARD_LINUX_FDT_ADDR>;
		};
EOF
		if [[ -n "$LINUX_INITRD" ]]; then
			cat <<EOF
		ramdisk {
			description = "initrd";
			data = /incbin/("$LINUX_INITRD");
			type = "ramdisk";
			arch = "arm64";
			os = "linux";
			compression = "none";
		};
EOF
		fi
		for overlay in "${LINUX_DTB_OVERLAYS[@]}"; do
			fullpath="${LINUX_DTB%/*}/$overlay"
			cat <<EOF
		$overlay {
			description = "$overlay";
			data = /incbin/("$fullpath.zst");
			type = "flat_dt";
			arch = "arm64";
			compression = "zstd";
			load = <$BOARD_LINUX_FDT_OVERLAY_ADDR>;
		};
EOF
		done
		cat <<EOF
	};
	configurations {
		default = "armadillo";
		armadillo {
			description = "armadillo";
			kernel = "kernel";
			fdt = "fdt";
EOF
		if [[ -n "$LINUX_INITRD" ]]; then
			cat <<EOF
			ramdisk = "ramdisk";
EOF
		fi
			cat <<EOF
		};
EOF
		for overlay in "${LINUX_DTB_OVERLAYS[@]}"; do
			cat <<EOF
		$overlay {
			fdt = "$overlay";
		};
EOF
		done
		cat <<EOF
	};
};
EOF
	} > "$TMPDIR/Image.its" \
		|| error "Could not write linux fit configuration"

	maybe_compress "$LINUX_IMAGE" "$TMPDIR/$dtb"
	for overlay in "${LINUX_DTB_OVERLAYS[@]}"; do
		maybe_compress "${LINUX_DTB%/*}/$overlay"
	done

	"$MKIMAGE" -f "$TMPDIR/Image.its" "$TMPDIR/Image.fit" \
		|| error "Could not build linux fit image"
}

build_initrd() {
	[ -e "$LINUX_INITRD" ] && return

	while [ -z "$LINUX_INITRD_TYPE" ]; do
		info "Select which initrd to build: [none|plain|mmc|verity]"
		to_stdout_only read -r LINUX_INITRD_TYPE
		case "$LINUX_INITRD_TYPE" in
		none|plain|mmc) ;;
		verity)
			[ -e "$LINUX_INITRD_VERITY_CERT" ] \
				|| error "verity initrd requires LINUX_INITRD_VERITY_CERT to be set";;
		*) LINUX_INITRD_TYPE="";;
		esac
	done

	local build_opt=( --board a"$BOARD_SERIES" )

	case "$LINUX_INITRD_TYPE" in
	none)
		LINUX_INITRD=""
		return
		;;
	plain)
		;;
	mmc)
		build_opt+=( "--lock" )
		;;
	verity)
		build_opt+=( --verity "$LINUX_INITRD_VERITY_CERT" )
		;;
	*)
		error "invalid LINUX_INITRD_TYPE $LINUX_INITRD_TYPE"
		;;
	esac

	LINUX_INITRD="$TMPDIR/initrd-baseos.$LINUX_INITRD_TYPE.zst"
	info "Building initrd for $LINUX_INITRD_TYPE (first time is slow)"
	(
		cd "$SECUREBOOT_DIR/build-rootfs" || exit
		./build_initrd.sh "$LINUX_INITRD" "${build_opt[@]}" || exit
		[ -e "$LINUX_INITRD" ]
	) || error "could not build initrd"
}

setup() {
	local build_rootfs_dir=""

	if ! [ -e "$IMXBOOT_DIR/.git" ]; then
		git -C "$IMXBOOT_DIR" init -b master -q
		git -C "$IMXBOOT_DIR" add .
		git -C "$IMXBOOT_DIR" commit -q -m "init from secureboot.sh setup"
	fi

	ln -sf "$IMXBOOT_DIR/secureboot.sh" "$SECUREBOOT_DIR/secureboot.sh" \
		|| error "symbolic link failed: secureboot.sh"

	if [ ! -e "$SECUREBOOT_DIR/secureboot.conf" ]; then
		cp "$IMXBOOT_DIR/secureboot.conf.example" "$SECUREBOOT_DIR/secureboot.conf" \
			|| error "could not create $SECUREBOOT_DIR/secureboot.conf"
	fi

	# no cst package
	if ! [ -e "$CST/linux64/bin/cst" ] && ! command -v cst > /dev/null; then
		info "imx-code-signing-tool package is needed to setup secureboot."
		prompt_yesno y "Install imx-code-signing-tool? [Y/n]" \
			|| error "Aborted by user"
		sudo apt install -y imx-code-signing-tool
	fi

	local build_rootfs_setup_dir="$SECUREBOOT_DIR/build-rootfs"
	if [ ! -e "$build_rootfs_setup_dir" ]; then
		build_rootfs_dir="$(find .. -maxdepth 1 -type d -name "build-rootfs-*" | sort -Vr | head -n 1)"
		# download latest if none found
		if [ -z "$build_rootfs_dir" ]; then
			info "Downloading latest build-rootfs..."
			curl -O https://download.atmark-techno.com/armadillo-iot-g4//tool/build-rootfs-latest.tar.gz \
				|| soft_error "failed to download: https://download.atmark-techno.com/armadillo-iot-g4//tool/build-rootfs-latest.tar.gz" \
			&& tar -C "$SECUREBOOT_DIR" -xaf build-rootfs-latest.tar.gz \
				|| soft_error "tar failed: build-rootfs-latest.tar.gz" \
			&& build_rootfs_dir="$SECUREBOOT_DIR/$(tar tf build-rootfs-latest.tar.gz \
					| awk -F '/' '{ print $1; exit; }')"
			git -C "$build_rootfs_dir" init -b master -q
			git -C "$build_rootfs_dir" add .
			git -C "$build_rootfs_dir" commit -q -m "init from secureboot.sh setup"
			rm -rf build-rootfs-latest.tar.gz
		fi

		# download failed?
		if [ -d "$build_rootfs_dir" ]; then
			build_rootfs_dir="$(realpath "$build_rootfs_dir")"
			info "OK: Linking $build_rootfs_setup_dir to $build_rootfs_dir"
			ln -sf "$build_rootfs_dir" "$build_rootfs_setup_dir" \
				|| soft_error "symbolic link failed: build_rootfs"
		elif [ -n "$build_rootfs_dir" ]; then
			soft_error "could not find $build_rootfs_dir"
		fi
	elif [ -L "$build_rootfs_setup_dir" ]; then
		info "OK: $build_rootfs_setup_dir is a symbolic link to $(readlink "$build_rootfs_setup_dir")"
	else
		info "OK: $build_rootfs_setup_dir exists"
	fi

	local imx_boot_setup_dir="$SECUREBOOT_DIR/imx-boot"
	if [ ! -e "$imx_boot_setup_dir" ]; then
		info "OK: Linking $imx_boot_setup_dir to $IMXBOOT_DIR"
		ln -sf "$IMXBOOT_DIR" "$imx_boot_setup_dir" \
			|| soft_error "symbolic link failed: imx-boot"
	elif [ -L "$imx_boot_setup_dir" ]; then
		info "OK: $imx_boot_setup_dir is a symbolic link to $(readlink "$imx_boot_setup_dir")"
	else
		info "OK: $imx_boot_setup_dir exists"
	fi

	command -v mkswu > /dev/null || soft_error "could not find mkswu command"

	[ -z "$ERRORS" ] || exit 1
	info "Setup $SECUREBOOT_DIR successfully. Run '$0 build' next."
}

download_linux_apk() {
	# $apk_dir must be set by parent
	local linux_apk="linux-at-x2-latest.apk"
	local apk_url="https://download.atmark-techno.com/armadillo-iot-g4/baseos/$linux_apk"
	local apk_file="$apk_dir/$linux_apk"

	mkdir -p "$apk_dir" || exit

	info "Downloading $apk_url"
	to_stdout curl -o "$apk_file" "$apk_url" \
		|| error "failed to download $apk_url"
	info

	tar --warning=no-unknown-keyword -xf "$apk_file" -C "$apk_dir" \
		|| error "Could not extract $apk_url"

	rm -f "$apk_file"

	# check package is sane
	[ -d "$apk_dir/boot" ] || error "could not find $apk_dir/boot"
	[ -d "$apk_dir/lib" ] || error "could not find $apk_dir/lib"
}

# This is verbose, so redirect logs early
check_or_download_linux() {
	local apk_dir="$TMPDIR/linux_apk"

	redirect_logs

	# use downloaded linux image/dtb/modules only if unset
	if ! [ -e "$LINUX_IMAGE" ]; then
		echo "Using linux kernel from apk"
		# modules must match image
		LINUX_IMAGE="$apk_dir/boot/Image"
		LINUX_MODULES="$apk_dir/lib/modules"
	fi
	if ! [ -e "$LINUX_DTB" ]; then
		echo "Using linux DTB from apk"
		LINUX_DTB="$apk_dir/boot/$BOARD_LINUX_DTB"
	fi

	if ! [ -e "$LINUX_IMAGE" ] || ! [ -e "$LINUX_DTB" ]; then
		download_linux_apk
	fi
	# ignore missing modules/initrd
	[ -e "$LINUX_MODULES" ] || LINUX_MODULES=""
	[ -e "$LINUX_INITRD" ] || LINUX_INITRD=""
	case "$LINUX_INITRD_TYPE" in
	""|none|plain|mmc) ;;
	verity)
		[ -n "$LINUX_INITRD_VERITY_CERT" ] \
			|| error "Please set LINUX_INITRD_VERITY_CERT"
		[ -e "$LINUX_INITRD_VERITY_CERT" ] \
			|| error "LINUX_INITRD_VERITY_CERT '$LINUX_INITRD_VERITY_CERT' not found"
		;;
	*) error "invalid LINUX_INITRD_TYPE: $LINUX_INITRD_TYPE (mustbe none/plain/mmc/verity)";;
	esac

	for overlay in "${LINUX_DTB_OVERLAYS[@]}" "${LINUX_DTB_OVERLAYS_PREAPPLY[@]}"; do
		if [[ "${overlay#/}" = "$overlay" ]]; then
			# relative to LINUX_DTB dir if no / in path
			overlay="${LINUX_DTB%/*}/$overlay"
		fi
		[[ -e "$overlay" ]] \
			|| error "$overlay not found in ${LINUX_DTB%/*}"
	done

	echo
	echo "The following files will be used in the subsequent processes."
	echo "image: $LINUX_IMAGE"
	echo "dtb: $LINUX_DTB"
	for dtbo in "${LINUX_DTB_OVERLAYS_PREAPPLY[@]}"; do
		if [[ "${dtbo#/}" != "$dtbo" ]]; then
			echo "dtb_preapplay: $dtbo"
		else
			echo "dtb_preapplay: ${LINUX_DTB%/*}/$dtbo"
		fi
	done
	for dtbo in "${LINUX_DTB_OVERLAYS[@]}"; do
		if [[ "${dtbo#/}" != "$dtbo" ]]; then
			echo "dtb: $dtbo"
		else
			echo "dtb: ${LINUX_DTB%/*}/$dtbo"
		fi
	done
	if [ -n "$LINUX_INITRD" ]; then
		echo "initrd: $LINUX_INITRD"
	elif [ "$LINUX_INITRD_TYPE" = none ]; then
		echo "initrd disabled"
	elif ! [ -e "$SECUREBOOT_DIR/build-rootfs" ]; then
		error "initrd not provided and build-rootfs does not exist, please run $0 setup"
	elif [ -n "$LINUX_INITRD_TYPE" ]; then
		echo "initrd: built for $LINUX_INITRD_TYPE"
	fi
}

download_installer_image() {
	local url=""
	local work_dir="$TMPDIR/installer.tmp"
	local zip="$work_dir/baseos-$BOARD_SERIES-installer-latest.zip"
	url="$BOARD_INSTALLER_URL_DIR/baseos-$BOARD_SERIES-installer-latest.zip"

	rm -rf "$work_dir" || exit
	mkdir -p "$work_dir" || exit

	info "Downloading $url"
	to_stdout curl -o "$zip" "$url" \
		|| error "Could not download $url"
	info

	unzip -d "$work_dir" "$zip"

	local img sig
	for img in "$work_dir"/*.img; do :; done
	for sig in "$work_dir"/*.sig; do :; done
	[ -e "$img" ] || error "image file not found in installer archive"

	if ! [ -e "$sig" ]; then
	       warning "signature file not found in installer archive"
	       prompt_yesno n "Continue anyway? [y/N]" \
		       || error "Aborted by user"

	       mv "$img" "$TMPDIR/"
	       rm -rf "$work_dir"
	       installer="$TMPDIR/${img##*/}"
	       return
	fi

	local ca_file="$SECUREBOOT_DIR/build-rootfs/common/image_firstboot/etc/swupdate.pem"
	[ -e "$ca_file" ] \
		|| error "swupdate.pem not found in $SECUREBOOT_DIR/common/image_firstboot/etc, cannot verify installer image -- aborting"

	openssl cms -verify -inform DER -in "$sig" \
			-content "$img" -nosmimecap -binary \
			-CAfile "$ca_file" >/dev/null \
		|| error "Signature verification failed"

	mv "$img" "$TMPDIR/"
	rm -rf "$work_dir"
	installer="$TMPDIR/${img##*/}"
}

find_or_download_installer() {
	local installer_img="baseos-$BOARD_SERIES-*.img"

	installer=$(find "$TMPDIR" -maxdepth 1 -type f -name "$installer_img" \
			| sort -Vr | head -n 1)

	if [ -e "$installer" ]; then
		echo "Re-using $installer"
		return
	fi

	download_installer_image
}

get_srk_fuse() {
	[ -e "$CST/crts/SRK_1_2_3_4_fuse.bin" ] \
		|| error "Secure boot key not initialized! Run $0 init_srk first"

	FUSES=()
	mapfile -t FUSES < <(hexdump -e '1/4 "0x"' -e '1/4 "%X""\n"' "$CST/crts/SRK_1_2_3_4_fuse.bin")
}

build() {
	set -o pipefail

	local resources_dir="$SECUREBOOT_DIR/tmp/resources"

	# prepare files for build linux kernel.
	echo "initrd: $LINUX_INITRD"
	echo

	# create srk hash if required
	init_srk
	info

	#write srk_hash to srk_hash.txt
	get_srk_fuse
	local srk_hash
	printf -v srk_hash "%s," "${FUSES[@]}"
	echo "${srk_hash%,}" > "$TMPDIR/srk_hash.txt"
	echo "created $TMPDIR/srk_hash.txt"
	echo
	# write to "CONFIG_ENV_WRITEABLE_LIST=y" to x2_defconfig.
	# setting "CONFIG_ENV_WRITEABLE_LIST=y" makes U-boot variables
	# without including in CFG_ENV_FLAGS_LIST_STATIC in armadillo_x2.h
	# unchangeble.

	local config="$IMXBOOT/uboot-imx/scripts/config"
	[ -e "$config" ] || config="$IMXBOOT/secureboot/uboot_config"

	# forbid environment modifications
	if ! grep -q "CONFIG_ENV_WRITEABLE_LIST=y" "$BOARD_ARMADILLO_DEFCONFIG"; then
		"$config" --file "$BOARD_ARMADILLO_DEFCONFIG" --enable ENV_WRITEABLE_LIST \
			|| error "could not set CONFIG_ENV_WRITEABLE_LIST=y in $BOARD_ARMADILLO_DEFCONFIG"
		echo "set CONFIG_ENV_WRITEABLE_LIST=y in $BOARD_ARMADILLO_DEFCONFIG"
		echo
	fi
	# write to "CONFIG_BOOTDELAY=-2" to defconfig.
	# setting "CONFIG_BOOTDELAY=-2" forbids entering boot prompt
	if ! grep -q "CONFIG_BOOTDELAY=-2" "$BOARD_ARMADILLO_DEFCONFIG"; then
		"$config" --file "$BOARD_ARMADILLO_DEFCONFIG" --set-val BOOTDELAY "-2" \
			|| error "could not set CONFIG_BOOTDELAY=-2 in $BOARD_ARMADILLO_DEFCONFIG"
		echo "set CONFIG_BOOTDELAY=-2 in $BOARD_ARMADILLO_DEFCONFIG"
		echo
	fi
	# disable boot script if board cannot do it itself
	if [ -n "$BOARD_ARMADILLO_H" ] && ! grep -q "loadbootscript=false" "$BOARD_ARMADILLO_H"; then
		sed -i -e 's/loadbootscript=.*\\0/loadbootscript=false\\0/' "$BOARD_ARMADILLO_H" \
			|| error "could not set loadbootscript=false in $BOARD_ARMADILLO_H"
		echo "set loadbootscript=false in $BOARD_ARMADILLO_H"
		echo
	fi

	# create signed boot loader
	imxboot
	[ -e "$OUT/imx-boot_$IMXBOOT_TARGET.signed" ] \
		|| error "$OUT/imx-boot_$IMXBOOT_TARGET.signed is not found"
	info

	# create signed linux kernel image
	rm -f "$OUT/Image.signed"
	# build without initrd for first phase
	LINUX_INITRD="" LINUX_INITRD_TYPE=none \
		linux
	[ -e "$OUT/Image.signed" ] || error "$OUT/Image.signed was not created as expected"
	info

	# init secureboot_x2/secureboot/resources directory.
	rm -rf "$resources_dir"
	mkdir -p "$resources_dir/lib"
	echo "$resources_dir initialized."

	# copy linux kernel image and others to resources directory.
	# resources dir will be included in SWU image.
	echo "copying following files to $resources_dir:"
	echo "resources/lib:"
	cp -r "$LINUX_MODULES" "$resources_dir/lib" \
		|| error "could not copy $LINUX_MODULES to $resources_dir/lib"
	echo " - $LINUX_MODULES"
	echo

	LINUX_OUT="$OUT/Image.signed-mmc"
	LINUX_INITRD_TYPE=mmc
	if [ -e "$LINUX_INITRD" ]; then
		if ! [[ "$LINUX_INITRD" = *.mmc.zst ]]; then
		       info "LINUX_INITRD '$LINUX_INITRD' is not locked to mmc."
		       prompt_yesno n "Continue anyway? [y/N]" \
			       || error "Aborted by user"
		       LINUX_OUT="$OUT/Image.signed"
		fi
	fi

	# build signed linux kernel image with locked initrd.
	rm -f "$LINUX_OUT"
	linux
	[ -e "$LINUX_OUT" ] || error "$LINUX_OUT was not created as expected"

	info

	# create SWU image to write srk hash and install linux kernel image and others
	info "Building 1_write_srk_install_kernel.swu..."
	DTBOS="${LINUX_DTB_OVERLAYS[*]##*/}" \
		mkswu "$IMXBOOT_DIR/secureboot/swu/1_write_srk_install_kernel.desc" \
			-o "$SWU_OUT_DIR/1_write_srk_install_kernel.swu" \
		|| error "could not create $SWU_OUT_DIR/1_write_srk_install_kernel.swu"
	echo

	# create SWU image to close secureboot
	info "Building 2_secureboot_close.swu..."
	mkswu "$IMXBOOT_DIR/secureboot/swu/2_secureboot_close.desc" \
			-o "$SWU_OUT_DIR/2_secureboot_close.swu" \
		|| error "could not create $SWU_OUT_DIR/2_secureboot_close.swu"

	echo

	#build SWU image to encrpt a strage on Armadillo.
	info "Building 3_disk_encryption.swu..."
	mkswu "$IMXBOOT_DIR/secureboot/swu/3_disk_encryption.desc" \
			-o "$SWU_OUT_DIR/3_disk_encryption.swu" \
		|| error "could not create $SWU_OUT_DIR/3_disk_encryption.swu"

	info
	info
	info "Please install SWU images in the following order:"
	info " - $SWU_OUT_DIR/1_write_srk_install_kernel.swu"
	info " - $SWU_OUT_DIR/2_secureboot_close.swu"
	info " - $SWU_OUT_DIR/3_disk_encryption.swu"
}

secureboot_init() {
	# for compat with older versions
	build
}

make_installer() {
	set -o pipefail

	local resources_dir="$TMPDIR/resources_verity"
	local cert_pem

	find_or_download_installer

	# create key.pem and cert.pem for verity check
	echo "creating key.pem and cert.pem"
	cert_pem="$TMPDIR/installer_verity_cert.pem"
	key_pem="$TMPDIR/installer_verity_key.pem"
	cn="${cn:-Secure boot installer $(\
		tr -dc "a-zA-Z0-9" < /dev/urandom| head -c 10)}"
	echo "use cn: $cn"
	echo
	openssl req -x509 -nodes -days 3650 -newkey ec \
		-pkeyopt ec_paramgen_curve:prime256v1 \
		-keyout "$key_pem" \
		-out "$cert_pem" \
		-subj /CN="$cn" \
		|| error "could not create $key_pem and $cert_pem"
	echo "created $cert_pem"
	echo "created $key_pem"
	echo

	# Always rebuild verity initrd (could not possibly guess the keys we just created)
	LINUX_INITRD_TYPE=verity
	LINUX_INITRD_VERITY_CERT="$cert_pem"
	LINUX_INITRD=""

	# sign verity kernel with that initrd
	rm -f "$OUT/Image.signed-verity"
	linux
	[ -e "$OUT/Image.signed-verity" ] || error "Verity image not created as expected"
	info

	# init secureboot_x2/secureboot/resources_verity directory.
	rm -rf "$resources_dir"
	mkdir -p "$resources_dir/sd_copy/boot" || exit
	cp "$OUT/Image.signed-verity" "$resources_dir/sd_copy/boot/Image" \
		|| error "could not copy $LINUX_INITRD to $resources_dir/sd_copy/boot/Image"
	cp "$cert_pem" "$key_pem" "$resources_dir" || exit

	# build SWU image to encrpt a strage on Armadillo.
	info "Building SWU..."
	echo "secureboot_make_installer.swu"
	INSTALLER_IMAGE="$installer" resources_dir="$resources_dir" \
		mkswu "$IMXBOOT_DIR/secureboot/swu/secureboot_make_installer.desc" \
			-o "$SWU_OUT_DIR/secureboot_make_installer.swu" \
		|| error "could not create $SWU_OUT_DIR/secureboot_make_installer.swu"
	info
	info
	info "Install following SWU image with an USB drive plugged in:"
	info " - $SWU_OUT_DIR/secureboot_make_installer.swu"
}

help() {
	echo "Usage: $0 action"
	echo
	echo "Where action is one of:"
	echo "init_srk: initialize super root key"
	echo "print_srk_fuse: Show uboot commands to burn fuses for generated key"
	echo "imxboot: build and sign imx-boot image"
	echo "imxboot_enc: build, encrypt and sign imx-boot image."
	echo "             Note this requires special handling to install."
	echo "linux: build a fit image from configured linux files and sign it."
	echo "       --image also can specify Linux kernel image path."
	echo "       --initrd also can specify locked initrd path."
	echo "       --dtb also can specify dtb path."
	echo "       --dtbo also can specify dtbo path."
	echo "setup: build an environment for secureboot setup."
	echo "build: build SWU images to enable secureboot and encrypt storage."
	echo "                 It takes the same arguments as linux, and:"
	echo "                 --modules also can specify modules path."
	echo "make_installer: build SWU images to make a installer disk image"
	echo "                for secureboot."
	echo "                It takes the same arguments as linux, besides the"
	echo "                listed arguments."
	echo "                --modules also can specify modules path."
	echo "                --cn also can specify CN for openssl. default is ten"
	echo "                random strings."
	echo
	echo "See $SECUREBOOT_DIR/secureboot.conf for configuration settings"
}

main() {
	local mode=""
	local arg=""
	local dtbo_seen=""

	(( $# == 0 )) && help

	while [ "$#" -ge 1 ]; do
		arg="$1"
		case "$arg" in
		imxboot|imxboot_enc|linux|linux_enc|init_srk|print_srk_fuse| \
		setup|secureboot_init|build|make_installer)
			mode="$arg"
			shift;;
		help|-h|--help) help; exit 0;;
		--image)
			[ "$#" -ge 2 ] || error "$arg requires an argument"
			LINUX_IMAGE="$2"
			shift 2
			;;
		--initrd)
			[ "$#" -ge 2 ] || error "$arg requires an argument"
			LINUX_INITRD="$2"
			shift 2
			;;
		--dtb)
			[ "$#" -ge 2 ] || error "$arg requires an argument"
			LINUX_DTB="$2"
			shift 2
			;;
		--dtbo)
			[ "$#" -ge 2 ] || error "$arg requires an argument"
			if [ -z "$dtbo_seen" ]; then
				dtbo_seen=1
				LINUX_DTB_OVERLAYS_PREAPPLY=()
				LINUX_DTB_OVERLAYS=()
			fi
			LINUX_DTB_OVERLAYS_PREAPPLY+=( "$2" )
			shift 2
			;;
		--modules)
			[ "$#" -ge 2 ] || error "$arg requires an argument"
			LINUX_MODULES="$2"
			shift 2
			;;
		--cn)
			[ "$#" -ge 2 ] || error "$arg requires an argument"
			case "$2" in
			*/*)
				error "--cn could not specify string included in '/' character."
				;;
			esac
			cn="$2"
			shift 2
			;;
		*) warning "Unrecognized option $arg"; echo; help; exit 1;;
		esac
	done

	pre_checks "$mode"

	"$mode"
}

main "$@"
