/* SPDX-License-Identifier: GPL-2.0+ */
/*
 * Copyright 2025 Atmark-techno
 */

#ifndef __ASM_ARCH_IMX8ULP_M33_PROTO_H
#define __ASM_ARCH_IMX8ULP_M33_PROTO_H

#include <asm/arch/sys_proto.h>

int mu_send_to_m33(u32 reg_index, u32 word);
int mu_recv_from_m33(u32 reg_index, ulong timeout_ms, u32 *word);

static inline int m33_send(u32 word)
{
	return mu_send_to_m33(0, word);
}

static inline int m33_send_many(const void *buf, int len)
{
	int rc, i;
	const u32 *word_buf = buf;

	for (i = 0; i < len / 4; i++) {
		rc = m33_send(word_buf[i]);
		if (rc)
			return rc;
	}

	if (len % 4) {
		const char *byte_buf = (const char *)(&word_buf[len / 4]);
		u32 last  = 0;

		for (i = 0; i < len % 4; i++) {
			last |= byte_buf[i] << (i * 8);
		}

		rc = m33_send(last);
		if (rc)
			return rc;
	}

	return 0;
}

static inline int m33_recv(ulong timeout_ms, u32 *word)
{
	return mu_recv_from_m33(0, timeout_ms, word);
}

/* note timeout is restarted for every word... */
static inline int m33_recv_many(ulong timeout_ms, void *buf, int len)
{
	int rc, i;
	u32 *word_buf = buf;

	for (i = 0; i < len / 4; i++) {
		rc = m33_recv(timeout_ms, word_buf + i);
		if (rc)
			return rc;
	}

	if (len % 4) {
		char *byte_buf = (char *)(&word_buf[len / 4]);
		u32 last;

		rc = m33_recv(timeout_ms, &last);
		if (rc)
			return rc;

		for (i = 0; i < len % 4; i++)  {
			byte_buf[i] = (last >> (i * 8)) & 0xff;
		}
	}

	return 0;
}

/* protocol for uboot <-> m33 */
/* handshake
 * uboot -> m33: handshake command
 * m33 -> uboot: handshake ACK
 */
#define M33_HANDSHAKE 0

/* boot into linux
 * uboot -> m33: boot command
 * (no reply)
 */
#define M33_BOOT 1

/* cold reset
 * uboot -> m33: reset command
 * (no reply)
 */
#define M33_RESET 2

/* pinctrl
 * uboot -> m33: pinctrl command, pinctrl values (u32 x6)
 * m33 -> uboot: ack
 * (Unlike linux's this is not registered anywhere on m33 side)
 */
#define M33_PINCTRL 3

/* I2C operations.
 * uboot -> m33: (i2c | (subcommand <<8) | (bus_id << 16))
 * rest depends on subcommand as follow. Anything else gets error back.
 */
#define M33_I2C 4
/* i2c init subcommand:
 * uboot -> m33: i2c type, index, baudrate
 * (type must be lpi2c, if we support lpuart later that one needs to send scl/sda too)
 * m33 -> uboot: status
 */
#define M33_I2C_INIT 0
/* i2c reset subcommand:
 * uboot -> m33: nothing else
 * m33 -> uboot: status
 */
#define M33_I2C_RESET 1
/* i2c read subcommand:
 * uboot -> m33: addr, len (in byte)
 * m33 -> uboot: status, actual read len (in byte), followed by data one word at a time
 */
#define M33_I2C_READ 2
/* i2c write subcommand:
 * uboot -> m33: addr, len (in byte), data one word at a time
 * m33 -> uboot: status
 */
#define M33_I2C_WRITE 3

/* debug console
 * uboot -> m33: this | (pin_tx << 8) | (pin_rx << 16)
 * m33 -> uboot: ack
 */
#define M33_DEBUG_CONSOLE 5

#endif

