// SPDX-License-Identifier: GPL-2.0+
/*
 * based on board/freescale/imx8ulp-evk/imx8ulp-evk.c
 *
 * Copyright (C) 2024 Atmark Techno, Inc. All Rights Reserved.
 */

#include <common.h>

#include <asm/arch/clock.h>
#include <asm/arch/imx8ulp-pins.h>
#include <asm/arch/m33_proto.h>
#include <asm/arch/m33_i2c.h>
#include <asm/arch/m33_iomuxc.h>
#include <asm/arch/pcc.h>
#include <asm/arch/sys_proto.h>
#include <asm/gpio.h>
#include <dt-bindings/power/imx8ulp-power.h>
#include <fuse.h>
#include <i2c.h>
#include <miiphy.h>
#include <miiphy.h>
#include <mmc.h>
#include <netdev.h>
#include <netdev.h>
#include <power-domain.h>

#include "board_kotai.h"

DECLARE_GLOBAL_DATA_PTR;

#define RTC_I2C_BUS 1
#define RTC_I2C_ADDR 0x32
#define RV8803_FLAG                     0x0E
#define RV8803_CTRL                     0x0F

#define RV8803_FLAG_AF                  BIT(3)
#define RV8803_FLAG_TF                  BIT(4)
#define RV8803_FLAG_UF                  BIT(5)

#define RV8803_CTRL_EIE                 BIT(2)
#define RV8803_CTRL_AIE                 BIT(3)
#define RV8803_CTRL_TIE                 BIT(4)
#define RV8803_CTRL_UIE                 BIT(5)

static void rtc_disarm_alarm(void)
{
	/* rv8803 interrupt line is connected to ONOFF signal (as well as gpio),
	 * so if we power up from rv8803 alarm the board will power back off
	 * after 5s unless we clear the alarm
	 */

	/* pinctrl from arch/arm64/boot/dts/freescale/armadillo_iotg_a9e.dts
	 * gpioa_iomuxc { lpi2c1 {}} */
	m33_pinctrl(IOMUXC_PTA4_LPI2C1_SCL, IOMUXC_PCR_ODE_MASK | IOMUXC_PCR_PE_MASK | IOMUXC_PCR_PS_MASK);
	m33_pinctrl(IOMUXC_PTA5_LPI2C1_SDA, IOMUXC_PCR_ODE_MASK | IOMUXC_PCR_PE_MASK | IOMUXC_PCR_PS_MASK);

	int ret;
	u8 ctrl, flag;

	ret = m33_i2c_init(0, RTC_I2C_BUS, 100000);
	if (ret) {
		printf("%s: Can't init i2c (%d)\n", __func__, ret);
		return;
	}

	ret = m33_i2c_read_reg(0, RTC_I2C_ADDR, RV8803_FLAG, &flag, 1);
	if (ret) {
		printf("%s i2c read flag failed, err %d\n", __func__, ret);
		m33_i2c_reset(0);
		return;
	}
	if (flag & (RV8803_FLAG_AF | RV8803_FLAG_TF | RV8803_FLAG_UF))
		printf("rv8803 rtc woken by interrupt\n");

	ret = m33_i2c_read_reg(0, RTC_I2C_ADDR, RV8803_CTRL, &ctrl, 1);
	if (ret) {
		printf("%s i2c read ctrl failed, err %d\n", __func__, ret);
		m33_i2c_reset(0);
		return;
	}
	ctrl &= ~(RV8803_CTRL_EIE | RV8803_CTRL_AIE | RV8803_CTRL_TIE | RV8803_CTRL_UIE);

	ret = m33_i2c_write_reg(0, RTC_I2C_ADDR, RV8803_CTRL, (const uint8_t *)&ctrl, 1);
	m33_i2c_reset(0);
	if (ret) {
		printf("%s i2c write ctrl failed, err %d\n", __func__, ret);
		return;
	}
	printf("Disabled RTC alarm\n");
}

static void check_board_rtc(void)
{
	u32 val;
	int ret;

	ret = fuse_read(EFUSE_SUBBOARD_BANK, EFUSE_SUBBOARD_WORD, &val);
	if (ret) {
		printf("Error: Cannot read eFuse\n");
		return;
	}

	val = val >> 16; //compare only the most significant 16bits
	switch (val) {
	case EFUSE_SUBBOARD_A9E_CAT1BIS_WLAN:
	case EFUSE_SUBBOARD_A9E_CAT1BIS:
	case EFUSE_SUBBOARD_A9E_WLAN:
	case EFUSE_SUBBOARD_A9E_LAN:
	case EFUSE_SUBBOARD_A900_EVA:
		rtc_disarm_alarm();
		break;
	}
}

void setup_m33_console(void)
{
	const char *m33_console = env_get("m33_console");
	char *s;
	u32 pin_tx, pin_rx, status;
	int rc;

	if (!m33_console || !strcmp(m33_console, "none"))
		return;

	pin_tx = simple_strtoul(m33_console, &s, 10);
	if (s[0] != '/')
		goto inval;
	pin_rx = simple_strtoul(s + 1, &s, 10);
	if (s[0] != 0 || pin_tx > 24 || pin_rx > 24)
		goto inval;

	rc = m33_send(M33_DEBUG_CONSOLE | (pin_tx << 8) | (pin_rx << 16));
	if (rc) {
		status = EIO;
		goto out;
	}

	rc = m33_recv(10, &status);
	if (rc)
		status = EIO;

out:
	if (status)
		printf("Setting m33 console failed for %u/%u: %u\n",
		       pin_tx, pin_rx, status);
	else
		printf("m33 console set to %u/%u\n", pin_tx, pin_rx);

	return;

inval:
	printf("m33_console must be 'none' or [tx]/[rx] (e.g. 18/19)\n");
}

#if IS_ENABLED(CONFIG_FEC_MXC)
#define ENET_CLK_PAD_CTRL	(PAD_CTL_PUS_UP | PAD_CTL_DSE | PAD_CTL_IBE_ENABLE)
static iomux_cfg_t const enet_clk_pads[] = {
	IMX8ULP_PAD_PTE19__ENET0_REFCLK | MUX_PAD_CTRL(ENET_CLK_PAD_CTRL),
};

static int setup_fec(void)
{
	/*
	 * Since ref clock and timestamp clock are from external,
	 * set the iomux prior the clock enablement
	 */
	imx8ulp_iomux_setup_multiple_pads(enet_clk_pads, ARRAY_SIZE(enet_clk_pads));

	/* Select enet time stamp clock: 001 - External Timestamp Clock */
	cgc1_enet_stamp_sel(1);

	/* enable FEC PCC */
	pcc_clock_enable(4, ENET_PCC4_SLOT, true);
	pcc_reset_peripheral(4, ENET_PCC4_SLOT, false);

	return 0;
}

int board_phy_config(struct phy_device *phydev)
{
	if (phydev->drv->config)
		phydev->drv->config(phydev);
	return 0;
}
#endif

int board_init(void)
{
	check_board_rtc();
#if defined(CONFIG_FEC_MXC)
	setup_fec();
#endif
	return 0;
}

int board_early_init_f(void)
{
	return 0;
}

static void set_serial(void)
{
	u32 product_id_val, lot_val;
	char serial[13 /* product (4), lot (8), \0 (1) */];
	char *old_serial;
	int ret;

	ret = fuse_read(EFUSE_KOTAI_BANK, EFUSE_KOTAI_WORD, &product_id_val);
	if (ret) {
		printf("Error: Can not read seihin kanri id\n");
		return;
	}

	ret = fuse_read(EFUSE_KOTAI_BANK, EFUSE_KOTAI_WORD + 1, &lot_val);
	if (ret) {
		printf("Error: Can not read lot number\n");
		return;
	}

	if (product_id_val == 0 && lot_val == 0) {
		printf("Error: serial number not written in fuses\n");
		return;
	}

	snprintf(serial, sizeof(serial), "%04X%08X", product_id_val, lot_val);

	old_serial = env_get("serial#");
	if (old_serial && old_serial[0] != '\0' && strcmp(serial, old_serial)) {
		/* board previously booted somewhere else, clear mac addresses */
		printf("serial# changed: Clearing MAC addresses in env\n");
		env_set("ethaddr", NULL);
		env_set("eth1addr", NULL);
	}

	env_set("serial#", serial);
}

int board_late_init(void)
{
#if CONFIG_IS_ENABLED(ENV_IS_IN_MMC)
	board_late_mmc_env_init();
#endif

	set_serial();
	setup_m33_console();
	return 0;
}

void board_quiesce_devices(void)
{
	/* Disable the power domains may used in u-boot before entering kernel */
#if CONFIG_IS_ENABLED(POWER_DOMAIN)
	struct udevice *scmi_devpd;
	int ret, i;
	struct power_domain pd;
	ulong ids[] = {
		IMX8ULP_PD_FLEXSPI2, IMX8ULP_PD_USB0, IMX8ULP_PD_USDHC0,
		IMX8ULP_PD_USDHC1, IMX8ULP_PD_USDHC2_USB1, IMX8ULP_PD_DCNANO,
		IMX8ULP_PD_MIPI_DSI};

	ret = uclass_get_device(UCLASS_POWER_DOMAIN, 0, &scmi_devpd);
	if (ret) {
		printf("Cannot get scmi devpd: err=%d\n", ret);
		return;
	}

	pd.dev = scmi_devpd;

	for (i = 0; i < ARRAY_SIZE(ids); i++) {
		pd.id = ids[i];
		ret = power_domain_off(&pd);
		if (ret)
			printf("power_domain_off %lu failed: err=%d\n", ids[i], ret);
	}
#endif
}
